@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,13 +51,11 @@ package com.openexchange.messaging.facebook;
 
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.EnumSet;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
-import com.google.code.facebookapi.FacebookException;
-import com.google.code.facebookapi.IFacebookRestClient;
-import com.google.code.facebookapi.ProfileField;
-import com.google.code.facebookapi.schema.UsersGetInfoResponse;
+import org.json.JSONException;
 import com.openexchange.messaging.DefaultMessagingFolder;
 import com.openexchange.messaging.DefaultMessagingPermission;
 import com.openexchange.messaging.MessagingAccount;
@@ -68,56 +66,80 @@ import com.openexchange.messaging.MessagingFolderAccess;
 import com.openexchange.messaging.MessagingPermission;
 import com.openexchange.messaging.Quota;
 import com.openexchange.messaging.Quota.Type;
+import com.openexchange.messaging.facebook.session.FacebookOAuthAccess;
 import com.openexchange.session.Session;
 
 /**
  * {@link FacebookMessagingFolderAccess} - The facebook folder access.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  * @since Open-Xchange v6.16
  */
-public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
+public class FacebookMessagingFolderAccess extends AbstractFacebookAccess implements MessagingFolderAccess {
 
-    private static String EMPTY = MessagingFolder.ROOT_FULLNAME;
+    private static interface FolderInfo {
+
+        public MessagingFolder generateFolder(FacebookMessagingFolderAccess folderAccess) throws MessagingException;
+
+        public MessagingFolder[] getSubfolders(FacebookMessagingFolderAccess folderAccess) throws MessagingException;
+
+        public MessagingFolder[] getPath(FacebookMessagingFolderAccess folderAccess) throws MessagingException;
+    }
+
+    private static final Map<String, FolderInfo> FOLDER_INFO_MAP;
+
+    static {
+        final Map<String, FolderInfo> m = new HashMap<String, FolderInfo>(2);
+
+        m.put(MessagingFolder.ROOT_FULLNAME, new FolderInfo() {
+
+            public MessagingFolder generateFolder(final FacebookMessagingFolderAccess folderAccess) throws MessagingException {
+                return folderAccess.getRootFolder();
+            }
+
+            public MessagingFolder[] getSubfolders(final FacebookMessagingFolderAccess folderAccess) throws MessagingException {
+                return new MessagingFolder[] { folderAccess.generateWallFolder() };
+            }
+
+            public MessagingFolder[] getPath(final FacebookMessagingFolderAccess folderAccess) throws MessagingException {
+                return EMPTY_PATH;
+            }
+        });
+        m.put(FacebookConstants.FOLDER_WALL, new FolderInfo() {
+
+            public MessagingFolder generateFolder(final FacebookMessagingFolderAccess folderAccess) throws MessagingException {
+                return folderAccess.generateWallFolder();
+            }
+
+            public MessagingFolder[] getSubfolders(final FacebookMessagingFolderAccess folderAccess) throws MessagingException {
+                return EMPTY_PATH;
+            }
+
+            public MessagingFolder[] getPath(final FacebookMessagingFolderAccess folderAccess) throws MessagingException {
+                return new MessagingFolder[] { folderAccess.generateWallFolder() };
+            }
+        });
+        FOLDER_INFO_MAP = Collections.unmodifiableMap(m);
+    }
 
     /**
      * The constant to return or represent an empty path.
      */
-    private static final MessagingFolder[] EMPTY_PATH = new MessagingFolder[0];
-
-    private final int id;
-
-    private final int user;
-
-    private final int cid;
-
-    private final IFacebookRestClient<Object> facebookRestClient;
-
-    private final long facebookUserId;
-
-    private final String facebookSession;
+    protected static final MessagingFolder[] EMPTY_PATH = new MessagingFolder[0];
 
     /**
      * Initializes a new {@link FacebookMessagingFolderAccess}.
-     * 
-     * @param facebookRestClient The facebook REST client
-     * @param messagingAccount The facebook messaging account
-     * @param session The session
-     * @param facebookUserId The facebook user identifier
-     * @param facebookSession The facebook session identifier
+     *
+     * @param facebookOAuthAccess The Facebook OAuth access
+     * @param messagingAccount The Facebook messaging account
+     * @param session The associated session
      */
-    public FacebookMessagingFolderAccess(final IFacebookRestClient<Object> facebookRestClient, final MessagingAccount messagingAccount, final Session session, final long facebookUserId, final String facebookSession) {
-        super();
-        this.facebookRestClient = facebookRestClient;
-        id = messagingAccount.getId();
-        user = session.getUserId();
-        cid = session.getContextId();
-        this.facebookUserId = facebookUserId;
-        this.facebookSession = facebookSession;
+    public FacebookMessagingFolderAccess(final FacebookOAuthAccess facebookOAuthAccess, final MessagingAccount messagingAccount, final Session session) {
+        super(facebookOAuthAccess, messagingAccount, session);
     }
 
     public void clearFolder(final String folderId) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
@@ -129,7 +151,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public void clearFolder(final String folderId, final boolean hardDelete) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
@@ -145,7 +167,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public String deleteFolder(final String folderId) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
@@ -157,7 +179,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public String deleteFolder(final String folderId, final boolean hardDelete) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
@@ -169,7 +191,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public boolean exists(final String folderId) throws MessagingException {
-        return (EMPTY.equals(folderId));
+        return (KNOWN_FOLDER_IDS.contains(folderId));
     }
 
     public String getConfirmedHamFolder() throws MessagingException {
@@ -185,7 +207,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public MessagingFolder getFolder(final String folderId) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
@@ -193,11 +215,11 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
                 Integer.valueOf(user),
                 Integer.valueOf(cid));
         }
-        return getRootFolder();
+        return FOLDER_INFO_MAP.get(folderId).generateFolder(this);
     }
 
     public Quota getMessageQuota(final String folderId) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
@@ -210,7 +232,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public MessagingFolder[] getPath2DefaultFolder(final String folderId) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
@@ -218,11 +240,11 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
                 Integer.valueOf(user),
                 Integer.valueOf(cid));
         }
-        return EMPTY_PATH;
+        return FOLDER_INFO_MAP.get(folderId).getPath(this);
     }
 
     public Quota[] getQuotas(final String folder, final Type[] types) throws MessagingException {
-        if (!EMPTY.equals(folder)) {
+        if (!KNOWN_FOLDER_IDS.contains(folder)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folder,
                 Integer.valueOf(id),
@@ -233,63 +255,79 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
         return Quota.getUnlimitedQuotas(types);
     }
 
-    private static final EnumSet<ProfileField> FIELDS_WALL_COUNT = EnumSet.of(ProfileField.WALL_COUNT);
-
     private static final Set<String> CAPS = Collections.emptySet();
 
     public MessagingFolder getRootFolder() throws MessagingException {
+        return generateRootFolder();
+    }
+
+    private MessagingFolder generateRootFolder() throws FacebookMessagingException {
         try {
             /*
              * The collection of users
              */
-            final UsersGetInfoResponse response =
-                (UsersGetInfoResponse) facebookRestClient.users_getInfo(
-                    Collections.singletonList(Long.valueOf(facebookUserId)),
-                    FIELDS_WALL_COUNT);
-            /*
-             * Fill root folder
-             */
-            final DefaultMessagingFolder dmf = new DefaultMessagingFolder();
-            dmf.setId(MessagingFolder.ROOT_FULLNAME);
-            dmf.setCapabilities(CAPS);
-            dmf.setDefaultFolder(false);
-            dmf.setDefaultFolderType(MessagingFolder.DefaultFolderType.NONE);
-            dmf.setDeletedMessageCount(0);
-            dmf.setExists(true);
-            dmf.setHoldsFolders(false);
-            dmf.setHoldsMessages(true);
-            dmf.setMessageCount(response.getUser().get(0).getWallCount().intValue());
-            dmf.setName("Facebook");
-            dmf.setNewMessageCount(0);
-
-            final DefaultMessagingPermission perm = DefaultMessagingPermission.newInstance();
-            perm.setAdmin(false);
-            perm.setAllPermissions(
-                MessagingPermission.READ_FOLDER,
-                MessagingPermission.READ_ALL_OBJECTS,
-                MessagingPermission.NO_PERMISSIONS,
-                MessagingPermission.NO_PERMISSIONS);
-            perm.setEntity(user);
-            perm.setGroup(false);
-
-            dmf.setOwnPermission(perm);
-            dmf.setParentId(null);
-            {
-                final List<MessagingPermission> perms = new ArrayList<MessagingPermission>(1);
-                perms.add(perm);
-                dmf.setPermissions(perms);
-            }
-            dmf.setRootFolder(true);
-            dmf.setSubfolders(false);
-            dmf.setSubscribed(true);
-            dmf.setSubscribedSubfolders(false);
-            dmf.setUnreadMessageCount(0);
-            return dmf;
-        } catch (final FacebookException e) {
-            throw FacebookMessagingException.create(e);
+            final int wallCount =
+                performFQLQuery(new StringBuilder("SELECT wall_count FROM user WHERE uid = ").append(facebookUserId).toString()).getInt(
+                    "wall_count");
+            final DefaultMessagingFolder rootFolder = generateFolder(MessagingFolder.ROOT_FULLNAME, null, "Facebook", wallCount);
+            rootFolder.setHoldsFolders(true);
+            rootFolder.setSubfolders(true);
+            rootFolder.setSubscribedSubfolders(true);
+            return rootFolder;
+        } catch (final JSONException e) {
+            throw FacebookMessagingExceptionCodes.JSON_ERROR.create(e, e.getMessage());
         }
     }
 
+    private static final int FQL_MAX_ROW_COUNT = 50;
+
+    protected MessagingFolder generateWallFolder() throws MessagingException {
+        final DefaultMessagingFolder wallFolder =
+            generateFolder(
+                FacebookConstants.FOLDER_WALL,
+                MessagingFolder.ROOT_FULLNAME,
+                I18n.getInstance().translate(getUserLocale(), NameStrings.NAME_WALL_FOLDER),
+                FQL_MAX_ROW_COUNT);
+        wallFolder.setRootFolder(false);
+        return wallFolder;
+    }
+
+    private DefaultMessagingFolder generateFolder(final String folderId, final String parentId, final String name, final int messageCount) {
+        final DefaultMessagingFolder dmf = new DefaultMessagingFolder();
+        dmf.setId(folderId);
+        dmf.setCapabilities(CAPS);
+        dmf.setDefaultFolder(false);
+        dmf.setDefaultFolderType(MessagingFolder.DefaultFolderType.NONE);
+        dmf.setDeletedMessageCount(0);
+        dmf.setExists(true);
+        dmf.setHoldsFolders(false);
+        dmf.setHoldsMessages(true);
+        dmf.setMessageCount(messageCount);
+        dmf.setName(name);
+        dmf.setNewMessageCount(0);
+
+        final DefaultMessagingPermission perm = DefaultMessagingPermission.newInstance();
+        perm.setAdmin(false);
+        final int[] arr = FacebookMessagingService.getStaticRootPerms();
+        perm.setAllPermissions(arr[0], arr[1], arr[2], arr[3]);
+        perm.setEntity(user);
+        perm.setGroup(false);
+
+        dmf.setOwnPermission(perm);
+        dmf.setParentId(parentId);
+        {
+            final List<MessagingPermission> perms = new ArrayList<MessagingPermission>(1);
+            perms.add(perm);
+            dmf.setPermissions(perms);
+        }
+        dmf.setRootFolder(true);
+        dmf.setSubfolders(false);
+        dmf.setSubscribed(true);
+        dmf.setSubscribedSubfolders(false);
+        dmf.setUnreadMessageCount(0);
+        return dmf;
+    }
+
     public String getSentFolder() throws MessagingException {
         return null;
     }
@@ -299,7 +337,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public Quota getStorageQuota(final String folderId) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
@@ -311,7 +349,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public MessagingFolder[] getSubfolders(final String parentIdentifier, final boolean all) throws MessagingException {
-        if (!EMPTY.equals(parentIdentifier)) {
+        if (!KNOWN_FOLDER_IDS.contains(parentIdentifier)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 parentIdentifier,
                 Integer.valueOf(id),
@@ -319,7 +357,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
                 Integer.valueOf(user),
                 Integer.valueOf(cid));
         }
-        return EMPTY_PATH;
+        return FOLDER_INFO_MAP.get(parentIdentifier).getSubfolders(this);
     }
 
     public String getTrashFolder() throws MessagingException {
@@ -327,7 +365,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public String moveFolder(final String folderId, final String newParentId) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
@@ -339,7 +377,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public String renameFolder(final String folderId, final String newName) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
@@ -351,7 +389,7 @@ public class FacebookMessagingFolderAccess implements MessagingFolderAccess {
     }
 
     public String updateFolder(final String folderId, final MessagingFolder toUpdate) throws MessagingException {
-        if (!EMPTY.equals(folderId)) {
+        if (!KNOWN_FOLDER_IDS.contains(folderId)) {
             throw MessagingExceptionCodes.FOLDER_NOT_FOUND.create(
                 folderId,
                 Integer.valueOf(id),
