@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,21 +49,20 @@
 
 package com.openexchange.messaging.facebook;
 
-import static com.openexchange.messaging.facebook.FormStrings.FORM_LABEL_LOGIN;
-import static com.openexchange.messaging.facebook.FormStrings.FORM_LABEL_PASSWORD;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
+import java.util.Set;
 import com.openexchange.datatypes.genericonf.DynamicFormDescription;
 import com.openexchange.datatypes.genericonf.FormElement;
+import com.openexchange.datatypes.genericonf.ReadOnlyDynamicFormDescription;
 import com.openexchange.messaging.MessagingAccountAccess;
 import com.openexchange.messaging.MessagingAccountManager;
 import com.openexchange.messaging.MessagingAccountTransport;
 import com.openexchange.messaging.MessagingAction;
 import com.openexchange.messaging.MessagingException;
+import com.openexchange.messaging.MessagingPermission;
 import com.openexchange.messaging.MessagingService;
-import com.openexchange.messaging.generic.DefaultMessagingAccountManager;
-import com.openexchange.messaging.generic.ReadOnlyDynamicFormDescription;
 import com.openexchange.session.Session;
 
 /**
@@ -100,14 +99,19 @@ public final class FacebookMessagingService implements MessagingService {
      */
     public FacebookMessagingService() {
         super();
-        accountManager = new DefaultMessagingAccountManager(ID);
+        accountManager = new FacebookMessagingAccountManager(this);
         final DynamicFormDescription tmpDescription = new DynamicFormDescription();
         /*
          * API & secret key
          */
-        tmpDescription.add(FormElement.input(FacebookConstants.FACEBOOK_LOGIN, FORM_LABEL_LOGIN, true, ""));
-        tmpDescription.add(FormElement.password(FacebookConstants.FACEBOOK_PASSWORD, FORM_LABEL_PASSWORD, true, ""));
-        this.formDescription = new ReadOnlyDynamicFormDescription(tmpDescription);
+        final FormElement oauthAccount = FormElement.custom("oauthAccount", "account", FormStrings.ACCOUNT_LABEL);
+        oauthAccount.setOption("type", "com.openexchange.oauth.facebook");
+        tmpDescription.add(oauthAccount);
+        formDescription = new ReadOnlyDynamicFormDescription(tmpDescription);
+    }
+
+    public Set<String> getSecretProperties() {
+        return Collections.emptySet();
     }
 
     public MessagingAccountAccess getAccountAccess(final int accountId, final Session session) throws MessagingException {
@@ -145,4 +149,16 @@ public final class FacebookMessagingService implements MessagingService {
          */
         return ACTIONS;
     }
+
+    public static int[] getStaticRootPerms() {
+        return new int[] {MessagingPermission.READ_FOLDER,
+            MessagingPermission.READ_ALL_OBJECTS,
+            MessagingPermission.NO_PERMISSIONS,
+            MessagingPermission.NO_PERMISSIONS};
+    }
+
+    public int[] getStaticRootPermissions() {
+        return getStaticRootPerms();
+    }
+
 }
