@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,14 +55,22 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
+import com.openexchange.html.HTMLService;
 import com.openexchange.messaging.MessagingException;
 import com.openexchange.messaging.MessagingHeader;
 import com.openexchange.messaging.StringContent;
+import com.openexchange.messaging.facebook.FacebookMessagingExceptionCodes;
 import com.openexchange.messaging.facebook.FacebookURLConnectionContent;
+import com.openexchange.messaging.facebook.services.FacebookMessagingServiceRegistry;
 import com.openexchange.messaging.facebook.utility.FacebookMessagingMessage;
 import com.openexchange.messaging.facebook.utility.FacebookMessagingUtility;
 import com.openexchange.messaging.generic.Utility;
@@ -72,10 +80,12 @@ import com.openexchange.messaging.generic.internet.MimeDateMessagingHeader;
 import com.openexchange.messaging.generic.internet.MimeMessagingBodyPart;
 import com.openexchange.messaging.generic.internet.MimeMultipartContent;
 import com.openexchange.messaging.generic.internet.MimeStringMessagingHeader;
+import com.openexchange.server.ServiceException;
+import com.openexchange.session.Session;
 
 /**
  * {@link FacebookFQLStreamParser} - Parses a given facebook stream element into a MIME message.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  * @since Open-Xchange v6.16
  */
@@ -119,7 +129,7 @@ public final class FacebookFQLStreamParser {
 
     private interface AttachmentHandler {
 
-        void handleAttachment(NodeList attachNodes, int len, StringBuilder messageText, MultipartProvider multipartProvider) throws MessagingException;
+        void handleAttachment(NodeList attachNodes, int len, FacebookMessagingMessage message, MultipartProvider multipartProvider) throws MessagingException;
     }
 
     private static final Map<String, ItemHandler> ITEM_HANDLERS;
@@ -142,12 +152,12 @@ public final class FacebookFQLStreamParser {
                 public void handleItem(final Node item, final FacebookMessagingMessage message) throws MessagingException {
                     // message.setHeader(MessagingHeader.KnownHeader.FROM.toString(), item.getTextContent());
                     final String content = item.getTextContent();
-                    final long fromUserId = FacebookMessagingUtility.parseUnsignedLong(null == content ? null : content.trim());
-                    if (fromUserId < 0) {
-                        org.apache.commons.logging.LogFactory.getLog(FacebookFQLStreamParser.class).warn(
+                    final long fromId = FacebookMessagingUtility.parseUnsignedLong(null == content ? null : content.trim());
+                    if (fromId < 0) {
+                        com.openexchange.log.Log.valueOf(LogFactory.getLog(FacebookFQLStreamParser.class)).warn(
                             new StringBuilder("Field actor_id cannot be parsed to a long: ``").append(content).append("\u00b4\u00b4").toString());
                     }
-                    message.setFromUserId(fromUserId);
+                    message.setFromId(fromId);
                 }
             });
             m.put("created_time", new ItemHandler() {
@@ -173,13 +183,22 @@ public final class FacebookFQLStreamParser {
                     }
                 }
             });
+            m.put("message", new ItemHandler() {
+
+                public void handleItem(final Node item, final FacebookMessagingMessage message) throws MessagingException {
+                    /*-
+                     * Text is already HTML-escaped: '<' ==> &lt;
+                     */
+                    message.appendTextContent(item.getTextContent());
+                }
+            });
             ITEM_HANDLERS = Collections.unmodifiableMap(m);
         }
 
-        final Map<String, AttachmentHandler> m = new HashMap<String, AttachmentHandler>(4);
-        m.put("link", new AttachmentHandler() {
+        final Map<String, AttachmentHandler> m = new HashMap<String, AttachmentHandler>(8);
+        m.put("album", new AttachmentHandler() {
 
-            public void handleAttachment(final NodeList attachNodes, final int len, final StringBuilder messageText, final MultipartProvider multipartProvider) {
+            public void handleAttachment(final NodeList attachNodes, final int len, final FacebookMessagingMessage message, final MultipartProvider multipartProvider) throws MessagingException {
                 String name = null;
                 String href = null;
                 int flag = 0;
@@ -194,6 +213,103 @@ public final class FacebookFQLStreamParser {
                     }
                 }
                 if (null != href && null != name) {
+                    final StringBuilder messageText = message.getMessageText();
+                    messageText.append(HTML_BR);
+                    final int pos = href.indexOf('?');
+                    if (pos < 0) {
+                        messageText.append(HTML_ANCHOR_START).append(href).append("'>").append(name).append(HTML_ANCHOR_END);
+                    } else {
+                        final int spos = href.indexOf("u=");
+                        final int epos = href.indexOf("&h=", spos + 2);
+                        if (epos < 0 || spos < 0) {
+                            messageText.append(HTML_ANCHOR_START).append(href).append("'>").append(name).append(HTML_ANCHOR_END);
+                        } else {
+                            /*-
+                             * Extract real URL from:
+                             * http://www.facebook.com/l.php?u=http%253A%252F%252Fwww.open-xchange.com&amp;h=f60781c95f3b41983168bbd0d0f52c95
+                             */
+                            try {
+                                messageText.append(HTML_ANCHOR_START).append(
+                                    URLDecoder.decode(URLDecoder.decode(href.substring(spos + 2, epos), "ISO-8859-1"), "ISO-8859-1")).append(
+                                    "'>").append(name).append(HTML_ANCHOR_END);
+                            } catch (final UnsupportedEncodingException e) {
+                                messageText.append(HTML_ANCHOR_START).append(href).append("'>").append(name).append(HTML_ANCHOR_END);
+                            }
+                        }
+                    }
+                }
+            }
+
+        });
+        m.put("link", new AttachmentHandler() {
+
+            public void handleAttachment(final NodeList attachNodes, final int len, final FacebookMessagingMessage message, final MultipartProvider multipartProvider) {
+                String name = null;
+                String href = null;
+                int flag = 0;
+                for (int i = 0; flag < 2 && i < len; i++) {
+                    final Node item = attachNodes.item(i);
+                    if ("name".equals(item.getLocalName())) {
+                        name = item.getTextContent();
+                        flag++;
+                    } else if ("href".equals(item.getLocalName())) {
+                        href = item.getTextContent();
+                        flag++;
+                    }
+                }
+                if (null != href && null != name) {
+                    final StringBuilder messageText = message.getMessageText();
+                    messageText.append(HTML_BR);
+                    final int pos = href.indexOf('?');
+                    if (pos < 0) {
+                        messageText.append(HTML_ANCHOR_START).append(href).append("'>").append(name).append(HTML_ANCHOR_END);
+                    } else {
+                        final int spos = href.indexOf("u=");
+                        final int epos = href.indexOf("&h=", spos + 2);
+                        if (epos < 0 || spos < 0) {
+                            messageText.append(HTML_ANCHOR_START).append(href).append("'>").append(name).append(HTML_ANCHOR_END);
+                        } else {
+                            /*-
+                             * Extract real URL from:
+                             * http://www.facebook.com/l.php?u=http%253A%252F%252Fwww.open-xchange.com&amp;h=f60781c95f3b41983168bbd0d0f52c95
+                             */
+                            try {
+                                messageText.append(HTML_ANCHOR_START).append(
+                                    URLDecoder.decode(URLDecoder.decode(href.substring(spos + 2, epos), "ISO-8859-1"), "ISO-8859-1")).append(
+                                    "'>").append(name).append(HTML_ANCHOR_END);
+                            } catch (final UnsupportedEncodingException e) {
+                                messageText.append(HTML_ANCHOR_START).append(href).append("'>").append(name).append(HTML_ANCHOR_END);
+                            }
+                        }
+                    }
+                }
+            } // End of handleAttachment
+        });
+        m.put("group", new AttachmentHandler() {
+
+            public void handleAttachment(final NodeList attachNodes, final int len, final FacebookMessagingMessage message, final MultipartProvider multipartProvider) throws MessagingException {
+                /*
+                 * A group post
+                 */
+                message.setGroup(true);
+                /*
+                 * Get group's link
+                 */
+                String name = null;
+                String href = null;
+                int flag = 0;
+                for (int i = 0; flag < 2 && i < len; i++) {
+                    final Node item = attachNodes.item(i);
+                    if ("name".equals(item.getLocalName())) {
+                        name = item.getTextContent();
+                        flag++;
+                    } else if ("href".equals(item.getLocalName())) {
+                        href = item.getTextContent();
+                        flag++;
+                    }
+                }
+                if (null != href && null != name) {
+                    final StringBuilder messageText = message.getMessageText();
                     messageText.append(HTML_BR);
                     final int pos = href.indexOf('?');
                     if (pos < 0) {
@@ -222,7 +338,7 @@ public final class FacebookFQLStreamParser {
         });
         m.put("video", new AttachmentHandler() {
 
-            public void handleAttachment(final NodeList attachNodes, final int len, final StringBuilder messageText, final MultipartProvider multipartProvider) throws MessagingException {
+            public void handleAttachment(final NodeList attachNodes, final int len, final FacebookMessagingMessage message, final MultipartProvider multipartProvider) throws MessagingException {
                 String sourceURL = null;
                 String ext = null;
                 /*
@@ -301,7 +417,7 @@ public final class FacebookFQLStreamParser {
         });
         m.put("photo", new AttachmentHandler() {
 
-            public void handleAttachment(final NodeList attachNodes, final int len, final StringBuilder messageText, final MultipartProvider multipartProvider) throws MessagingException {
+            public void handleAttachment(final NodeList attachNodes, final int len, final FacebookMessagingMessage message, final MultipartProvider multipartProvider) throws MessagingException {
                 String sourceURL = null;
                 String ext = null;
                 /*
@@ -337,6 +453,7 @@ public final class FacebookFQLStreamParser {
                     /*
                      * TODO: Add as multipart/related?!?!
                      */
+                    final StringBuilder messageText = message.getMessageText();
                     messageText.append(HTML_BR);
                     messageText.append("<img src='").append(sourceURL);
                     if (null == ext) {
@@ -347,12 +464,75 @@ public final class FacebookFQLStreamParser {
                 }
             }
         });
+        m.put("swf", new AttachmentHandler() {
+
+            public void handleAttachment(final NodeList attachNodes, final int len, final FacebookMessagingMessage message, final MultipartProvider multipartProvider) throws MessagingException {
+                String sourceURL = null;
+                /*
+                 * "media" node
+                 */
+                final Node media = getNodeByName("media", attachNodes, len);
+                if (null != media) {
+                    final Node streamMedia = getNodeByName("stream_media", media);
+                    if (null != streamMedia) {
+                        final Node swf = getNodeByName("swf", streamMedia);
+                        if (null != swf) {
+                            /*
+                             * "source_url" node
+                             */
+                            final Node src = getNodeByName("source_url", swf);
+                            if (null != src) {
+                                sourceURL = src.getTextContent();
+                            }
+                        }
+                    }
+                }
+                String nameStr = null;
+                /*
+                 * "name" node
+                 */
+                final Node name = getNodeByName("name", attachNodes, len);
+                if (null != name) {
+                    nameStr = name.getTextContent();
+                }
+                /*
+                 * Source URL present?
+                 */
+                if (null != sourceURL && null != nameStr) {
+                    final StringBuilder messageText = message.getMessageText();
+                    messageText.append(HTML_BR);
+                    final int pos = sourceURL.indexOf('?');
+                    if (pos < 0) {
+                        messageText.append(HTML_ANCHOR_START).append(sourceURL).append("'>").append(nameStr).append(HTML_ANCHOR_END);
+                    } else {
+                        final int spos = sourceURL.indexOf("u=");
+                        final int epos = sourceURL.indexOf("&h=", spos + 2);
+                        if (epos < 0 || spos < 0) {
+                            messageText.append(HTML_ANCHOR_START).append(sourceURL).append("'>").append(nameStr).append(HTML_ANCHOR_END);
+                        } else {
+                            /*-
+                             * Extract real URL from:
+                             * http://www.facebook.com/l.php?u=http%253A%252F%252Fwww.open-xchange.com&amp;h=f60781c95f3b41983168bbd0d0f52c95
+                             */
+                            try {
+                                messageText.append(HTML_ANCHOR_START).append(
+                                    URLDecoder.decode(URLDecoder.decode(sourceURL.substring(spos + 2, epos), "ISO-8859-1"), "ISO-8859-1")).append(
+                                    "'>").append(nameStr).append(HTML_ANCHOR_END);
+                            } catch (final UnsupportedEncodingException e) {
+                                messageText.append(HTML_ANCHOR_START).append(sourceURL).append("'>").append(nameStr).append(HTML_ANCHOR_END);
+                            }
+                        }
+                    }
+                }
+            }
+        });
         m.put("event", new AttachmentHandler() {
 
-            public void handleAttachment(final NodeList attachNodes, final int len, final StringBuilder messageText, final MultipartProvider multipartProvider) throws MessagingException {
+            public void handleAttachment(final NodeList attachNodes, final int len, final FacebookMessagingMessage message, final MultipartProvider multipartProvider) throws MessagingException {
 
                 final Node name = getNodeByName("name", attachNodes, len);
                 if (null != name) {
+                    final StringBuilder messageText = message.getMessageText();
                     final Node href = getNodeByName("href", attachNodes, len);
                     if (null == href) {
                         messageText.append(HTML_BR).append(name.getTextContent());
@@ -365,8 +545,11 @@ public final class FacebookFQLStreamParser {
                     if (null != properties) {
                         final List<Node> streamProps = getNodesByName("stream_property", properties);
                         for (final Node streamProperty : streamProps) {
-                            messageText.append(HTML_BR).append(getNodeByName("name", streamProperty).getTextContent());
-                            messageText.append(": ").append(getNodeByName("text", streamProperty).getTextContent());
+                            final Node nameProperty = getNodeByName("name", streamProperty);
+                            if (null != nameProperty) {
+                                messageText.append(HTML_BR).append(nameProperty.getTextContent());
+                                messageText.append(": ").append(getNodeByName("text", streamProperty).getTextContent());
+                            }
                         }
                     }
                 }
@@ -378,21 +561,23 @@ public final class FacebookFQLStreamParser {
 
     /**
      * Parses given facebook stream element into a MIME message.
-     * 
+     *
      * @param streamElement The facebook stream element
+     * @param locale The user's locale
      * @return The resulting MIME message
      * @throws MessagingException If parsing fails
      */
-    public static FacebookMessagingMessage parseStreamDOMElement(final Element streamElement) throws MessagingException {
+    public static FacebookMessagingMessage parseStreamDOMElement(final Element streamElement, final Locale locale, final Session session) throws MessagingException {
         if (!streamElement.hasChildNodes()) {
             return null;
         }
-        final FacebookMessagingMessage message = new FacebookMessagingMessage();
+        checkFacebookError(streamElement);
+        final FacebookMessagingMessage message = new FacebookMessagingMessage(locale);
         /*
          * Iterate child nodes
          */
+        boolean anyHandlerFound = false;
         Node attachmentNode = null;
-        final StringBuilder messageText = new StringBuilder(256);
         {
             final NodeList childNodes = streamElement.getChildNodes();
             final int len = childNodes.getLength();
@@ -402,19 +587,14 @@ public final class FacebookFQLStreamParser {
                 if (null != localName) {
                     final ItemHandler itemHandler = ITEM_HANDLERS.get(localName);
                     if (null == itemHandler) {
-                        if ("message".equals(localName)) {
-                            /*-
-                             * Text is already HTML-escaped: '<' ==> &lt;
-                             */
-                            messageText.append(item.getTextContent());
-                        } else if ("attachment".equals(localName)) {
+                        if ("attachment".equals(localName)) {
                             attachmentNode = item;
                         } else {
-                            org.apache.commons.logging.LogFactory.getLog(FacebookFQLStreamParser.class).warn(
-                                "Un-handled item: " + localName);
+                            com.openexchange.log.Log.valueOf(LogFactory.getLog(FacebookFQLStreamParser.class)).warn("Un-handled item: " + localName);
                         }
                     } else {
                         itemHandler.handleItem(item, message);
+                        anyHandlerFound = true;
                     }
                 }
             }
@@ -422,14 +602,18 @@ public final class FacebookFQLStreamParser {
         /*
          * Add subject & size
          */
-        message.setHeader(new MimeStringMessagingHeader(
-            MessagingHeader.KnownHeader.SUBJECT.toString(),
-            FacebookMessagingUtility.abbreviate(messageText.toString(), 140)));
-        message.setSize(messageText.length());
+        if (!anyHandlerFound && null == attachmentNode) {
+            /*
+             * Empty message... ignore it!
+             */
+            com.openexchange.log.Log.valueOf(LogFactory.getLog(FacebookFQLStreamParser.class)).warn("Empty Facebook message detected:\n" + streamElement);
+            return null;
+        }
         /*
          * Check attachment node
          */
         final MultipartProvider multipartProvider = new MultipartProvider();
+        boolean attachmentHandlerFound = false;
         if (null != attachmentNode && attachmentNode.hasChildNodes()) {
             final NodeList attachmentChildNodes = attachmentNode.getChildNodes();
             final int len = attachmentChildNodes.getLength();
@@ -443,39 +627,173 @@ public final class FacebookFQLStreamParser {
                         final String type = item.getTextContent();
                         final AttachmentHandler attachmentHandler = ATTACH_HANDLERS.get(type);
                         if (null == attachmentHandler) {
-                            org.apache.commons.logging.LogFactory.getLog(FacebookFQLStreamParser.class).warn(
-                                "Unknown attachment type: " + type);
+                            final Log logger = com.openexchange.log.Log.valueOf(LogFactory.getLog(FacebookFQLStreamParser.class));
+                            logger.warn("Unknown attachment type: " + type);
+                            logger.debug("Stream element:\n" + Utility.prettyPrintXML(streamElement));
                         } else {
-                            attachmentHandler.handleAttachment(attachmentChildNodes, len, messageText, multipartProvider);
+                            attachmentHandler.handleAttachment(attachmentChildNodes, len, message, multipartProvider);
                         }
+                        attachmentHandlerFound = true;
                         break;
                     }
                     /*
                      * A link attachment; append it to message
                      */
                     final AttachmentHandler attachmentHandler = ATTACH_HANDLERS.get("link");
-                    attachmentHandler.handleAttachment(attachmentChildNodes, len, messageText, multipartProvider);
+                    attachmentHandler.handleAttachment(attachmentChildNodes, len, message, multipartProvider);
+                    attachmentHandlerFound = true;
                     break;
                 }
             }
+            if (!attachmentHandlerFound) {
+                final Node media = getNodeByName("media", attachmentChildNodes, len);
+                if (null != media) {
+                    final Node streamMedia = getNodeByName("stream_media", media);
+                    if (null != streamMedia) {
+                        final Node type = getNodeByName("type", streamMedia);
+                        if (null != type) {
+                            final String sType = type.getTextContent();
+                            final AttachmentHandler attachmentHandler = ATTACH_HANDLERS.get(sType);
+                            if (null == attachmentHandler) {
+                                final Log logger = com.openexchange.log.Log.valueOf(LogFactory.getLog(FacebookFQLStreamParser.class));
+                                logger.warn("Unknown attachment type: " + sType);
+                                logger.debug("Stream element:\n" + Utility.prettyPrintXML(streamElement));
+                            } else {
+                                attachmentHandler.handleAttachment(attachmentChildNodes, len, message, multipartProvider);
+                            }
+                            attachmentHandlerFound = true;
+                        }
+                    }
+                }
+            }
         }
-
+        /*
+         * Set subject & size
+         */
+        final StringBuilder messageText = message.getMessageText();
+        final int size = messageText.length();
+        /*
+         * Debug
+         */
+        if (size <= 0 && !attachmentHandlerFound) {
+            final Log logger = com.openexchange.log.Log.valueOf(LogFactory.getLog(FacebookFQLStreamParser.class));
+            logger.debug("Stream element lead to empty message:\n" + Utility.prettyPrintXML(streamElement));
+        }
+        message.setSize(size);
+        final String htmlContent;
+        final String preparedContent;
+        try {
+            String tmp = messageText.toString();
+            final HTMLService service = FacebookMessagingServiceRegistry.getServiceRegistry().getService(HTMLService.class, true);
+            htmlContent = service.replaceImages(tmp, session.getSessionID());
+            tmp = replaceImages(tmp);
+            preparedContent = service.replaceImages(tmp, session.getSessionID());
+        } catch (final ServiceException e) {
+            throw new MessagingException(e);
+        }
+        final String subject = FacebookMessagingUtility.abbreviate(htmlContent, 140);
+        message.setHeader(new MimeStringMessagingHeader(MessagingHeader.KnownHeader.SUBJECT.toString(), subject));
+        message.setToString(subject);
+        /*
+         * Set content
+         */
         final MimeMultipartContent multipartContent = multipartProvider.getMultipartContent();
         if (null == multipartContent) {
-            message.setContent(createAlternative(messageText.toString()));
+            message.setContent(createAlternative(htmlContent, preparedContent));
         } else {
             final MimeMessagingBodyPart altBodyPart = new MimeMessagingBodyPart();
-            altBodyPart.setContent(createAlternative(messageText.toString()));
+            altBodyPart.setContent(createAlternative(htmlContent, preparedContent));
             multipartContent.addBodyPart(altBodyPart, 0);
             message.setContent(multipartContent);
         }
-
+        /*
+         * Return parsed message
+         */
         return message;
     }
 
+    private static final Pattern IMG_PATTERN = Pattern.compile("<img[^>]*>", Pattern.CASE_INSENSITIVE | Pattern.DOTALL);
+
+    private static String replaceImages(final String content) {
+        if (null == content) {
+            return null;
+        }
+        try {
+            final Matcher imgMatcher = IMG_PATTERN.matcher(content);
+            if (imgMatcher.find()) {
+                /*
+                 * Start replacing with href
+                 */
+                final StringBuilder sb = new StringBuilder(content.length());
+                int lastMatch = 0;
+                do {
+                    sb.append(content.substring(lastMatch, imgMatcher.start()));
+                    final String imgTag = imgMatcher.group();
+                    replaceWithSrcAttribute(imgTag, sb);
+                    lastMatch = imgMatcher.end();
+                } while (imgMatcher.find());
+                sb.append(content.substring(lastMatch));
+                return sb.toString();
+            }
+
+        } catch (final Exception e) {
+            // Ignore
+        }
+        return content;
+    }
+
+    private static final Pattern SRC_PATTERN = Pattern.compile(
+        "(?:src=\"([^\"]*)\")|(?:src='([^']*)')|(?:src=[^\"']([^\\s>]*))",
+        Pattern.CASE_INSENSITIVE | Pattern.DOTALL);
+
+    private static void replaceWithSrcAttribute(final String imgTag, final StringBuilder sb) {
+        final Matcher srcMatcher = SRC_PATTERN.matcher(imgTag);
+        if (srcMatcher.find()) {
+            /*
+             * Extract URL
+             */
+            int group = 1;
+            String urlStr = srcMatcher.group(group);
+            if (urlStr == null) {
+                urlStr = srcMatcher.group(++group);
+                if (urlStr == null) {
+                    urlStr = srcMatcher.group(++group);
+                }
+            }
+            if (urlStr == null) {
+                sb.append(imgTag);
+            } else {
+                sb.append(urlStr);
+            }
+        } else {
+            sb.append(imgTag);
+        }
+    }
+
+    private static void checkFacebookError(final Element streamElement) throws MessagingException {
+        long errorCode = -1L;
+        String errorMsg = null;
+        final NodeList childNodes = streamElement.getChildNodes();
+        final int length = childNodes.getLength();
+        for (int i = 0; i < length; i++) {
+            final Node item = childNodes.item(i);
+            final String localName = item.getLocalName();
+            if (null != localName) {
+                if ("error_code".equals(localName)) {
+                    errorCode = FacebookMessagingUtility.parseUnsignedLong(item.getTextContent());
+                } else if ("error_msg".equals(localName)) {
+                    errorMsg = item.getTextContent();
+                }
+            }
+        }
+        if (errorCode >= 0) {
+            throw FacebookMessagingExceptionCodes.FB_API_ERROR.create(Long.valueOf(errorCode), null == errorMsg ? "" : errorMsg);
+        }
+    }
+
     /**
      * Gets the first node with specified name occurring as child below given node.
-     * 
+     *
      * @param name The node name to look-up
      * @param node The parent node
      * @return The appropriate node or <code>null</code> if none found
@@ -490,7 +808,7 @@ public final class FacebookFQLStreamParser {
 
     /**
      * Gets the first node with specified name occurring in given node list.
-     * 
+     *
      * @param name The node name to look-up
      * @param nodes The node list
      * @param len The length of the node list
@@ -508,7 +826,7 @@ public final class FacebookFQLStreamParser {
 
     /**
      * Gets all nodes with specified name occurring as child below given node.
-     * 
+     *
      * @param name The node name to look-up
      * @param node The parent node
      * @return The appropriate nodes
@@ -533,11 +851,11 @@ public final class FacebookFQLStreamParser {
 
     private static final String UTF_8 = "UTF-8";
 
-    private static MimeMultipartContent createAlternative(final String messageText) throws MessagingException {
+    private static MimeMultipartContent createAlternative(final String messageText, final String altText) throws MessagingException {
         final MimeMultipartContent alt = new MimeMultipartContent("alternative");
         {
             final MimeMessagingBodyPart text = new MimeMessagingBodyPart();
-            text.setText(Utility.textFormat(messageText), UTF_8, "plain");
+            text.setText(Utility.textFormat(null == altText ? messageText : altText), UTF_8, "plain");
             text.setHeader("MIME-Version", "1.0");
             text.setHeader("Content-Type", "text/plain; charset=UTF-8");
             alt.addBodyPart(text);
